/*  lil-gp Genetic Programming System, version 1.0, 11 July 1995
 *  Copyright (C) 1995  Michigan State University
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of version 2 of the GNU General Public License as
 *  published by the Free Software Foundation.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  
 *  Douglas Zongker       (zongker@isl.cps.msu.edu)
 *  Dr. Bill Punch        (punch@isl.cps.msu.edu)
 *
 *  Computer Science Department
 *  A-714 Wells Hall
 *  Michigan State University
 *  East Lansing, Michigan  48824
 *  USA
 *  
 */

#include <stdio.h>
#include <math.h>
#include <lilgp.h>
#include "defines.h"
#include "app.h"

//CUDA includee
#include <cuda.h>
#include <cuda_runtime.h>

//external methods for CUDA outside of this file.
extern void doTest();
//evaluated the population on the gpu
extern void eval_pop_gpu(char* inds, unsigned long int indsSize, int numTrees, char *signals, unsigned long int popSize, float*roi,  unsigned long int * startInds, int compareToSeq);
//copies the indicators to the gpu
extern void copy_init_data(float *fInds, char *bInds);
//released memory from the gpu after run is complete
extern void uninit_data();

//File to write output too and the test number of the current run
extern FILE * myOutputFile;
extern int testRunNum;

static int fitness_cases = -1;
static double *app_fitness_cases[2];
static double value_cutoff;

//keeps track of the maximum difference in roi found when comparing GPU to sequential evaluation
//used when checkingto make sure hh GPU is returning the same results as the sequential would
double maxDiff = 0;


//Names of files to read stock info from
//char *STOCK_NAMES[] =  {"BAC.csv", "GOOG.csv", "ALL.csv", "BA.csv", "BAM.csv", "BCE.csv", "BMO.csv", "CM.csv", "CSCO.csv", "CTL.csv", "DO.csv", "GE.csv", "IBM.csv", "JNJ.csv", "JPM.csv", "KO.csv", "LMT.csv", "MCD.csv", "MO.csv", "MSFT.csv", "NSRGY.csv", "PEP.csv", "PG.csv", "RAI.csv", "RCI.csv", "T.csv", "TD.csv", "TRP.csv", "UVV.csv", "VZ.csv", "WMT.csv", "XOM.csv", "XRX.csv"};
char *STOCK_NAMES[] =  {"Stocks/AAPL.csv", "Stocks/ABM.csv", "Stocks/ACAT.csv", "Stocks/ADM.csv", "Stocks/ADSK.csv", "Stocks/AEE.csv", "Stocks/AEO.csv", "Stocks/AES.csv", "Stocks/AFL.csv", "Stocks/AGN.csv", "Stocks/ALL.csv", "Stocks/AMGN.csv", "Stocks/AMZN.csv", "Stocks/ANF.csv", "Stocks/ARW.csv", "Stocks/ATVI.csv", "Stocks/AXP.csv", "Stocks/BA.csv", "Stocks/BAC.csv", "Stocks/BAM.csv", "Stocks/BBBY.csv", "Stocks/BBY.csv", "Stocks/BCE.csv", "Stocks/BHI.csv", "Stocks/BKS.csv", "Stocks/BLOKA.csv", "Stocks/BMO.csv", "Stocks/BRCM.csv", "Stocks/BUD.csv", "Stocks/CM.csv", "Stocks/CMCSA.csv", "Stocks/COF.csv", "Stocks/CSCO.csv", "Stocks/CTL.csv", "Stocks/CVS.csv", "Stocks/DE.csv", "Stocks/DELL.csv", "Stocks/DIS.csv", "Stocks/DISH.csv", "Stocks/DO.csv", "Stocks/DOW.csv", "Stocks/EBAY.csv", "Stocks/ECL.csv", "Stocks/EMN.csv", "Stocks/EMR.csv", "Stocks/F.csv", "Stocks/FDX.csv", "Stocks/GE.csv", "Stocks/GIS.csv", "Stocks/GT.csv", "Stocks/HAL.csv", "Stocks/HAS.csv", "Stocks/HD.csv", "Stocks/HOG.csv", "Stocks/HON.csv", "Stocks/HRB.csv", "Stocks/IBM.csv", "Stocks/JCP.csv", "Stocks/JNJ.csv", "Stocks/JPM.csv", "Stocks/K.csv", "Stocks/KO.csv", "Stocks/KR.csv", "Stocks/LMT.csv", "Stocks/LOW.csv", "Stocks/LPX.csv", "Stocks/LUV.csv", "Stocks/MCD.csv", "Stocks/MLM.csv", "Stocks/MLR.csv", "Stocks/MMM.csv", "Stocks/MO.csv", "Stocks/MOT.csv", "Stocks/MRO.csv", "Stocks/MS.csv", "Stocks/MSFT.csv", "Stocks/MSO.csv", "Stocks/NCR.csv", "Stocks/NKE.csv", "Stocks/NOC.csv", "Stocks/NOVL.csv", "Stocks/NSRGY.csv", "Stocks/NTAP.csv", "Stocks/NVLS.csv", "Stocks/ODP.csv", "Stocks/ORCL.csv", "Stocks/PEP.csv", "Stocks/PFE.csv", "Stocks/PG.csv", "Stocks/PII.csv", "Stocks/Q.csv", "Stocks/RAI.csv", "Stocks/RCI.csv", "Stocks/RGS.csv", "Stocks/RHT.csv", "Stocks/RSH.csv", "Stocks/S.csv", "Stocks/SALM.csv", "Stocks/SBUX.csv", "Stocks/SIX.csv", "Stocks/SNA.csv", "Stocks/SNE.csv", "Stocks/SPLS.csv", "Stocks/STEI.csv", "Stocks/STSA.csv", "Stocks/STT.csv", "Stocks/SUN.csv", "Stocks/SWY.csv", "Stocks/T.csv", "Stocks/TAP.csv", "Stocks/TD.csv", "Stocks/TGI.csv", "Stocks/TGT.csv", "Stocks/TRN.csv", "Stocks/TRP.csv", "Stocks/TXN.csv", "Stocks/ULTI.csv", "Stocks/UPS.csv", "Stocks/USB.csv", "Stocks/UTX.csv", "Stocks/UVV.csv", "Stocks/VLO.csv", "Stocks/VZ.csv", "Stocks/WAG.csv", "Stocks/WEN.csv", "Stocks/WFMI.csv", "Stocks/WMT.csv", "Stocks/WWE.csv", "Stocks/XLNX.csv", "Stocks/XOM.csv", "Stocks/XRX.csv", "Stocks/YHOO.csv", "Stocks/YUM.csv"};
//char *STOCK_NAMES[] =  {"BAC.csv", "Stocks/GOOG.csv", "Stocks/ALL.csv", "Stocks/BA.csv", "Stocks/BAM.csv", "Stocks/BCE.csv", "Stocks/BMO.csv", "Stocks/CM.csv", "Stocks/CSCO.csv", "Stocks/CTL.csv", "Stocks/DO.csv", "Stocks/GE.csv", "Stocks/IBM.csv", "Stocks/JNJ.csv", "Stocks/JPM.csv", "Stocks/KO.csv", "Stocks/LMT.csv", "Stocks/MCD.csv", "Stocks/MO.csv", "Stocks/MSFT.csv", "Stocks/NSRGY.csv", "Stocks/PEP.csv", "Stocks/PG.csv", "Stocks/RAI.csv", "Stocks/RCI.csv", "Stocks/T.csv", "Stocks/TD.csv", "Stocks/TRP.csv", "Stocks/UVV.csv", "Stocks/VZ.csv", "Stocks/WMT.csv", "Stocks/XOM.csv", "Stocks/XRX.csv"};

//Stores information about a specific stock/day
StockDay stockDays[NUM_STOCKS][COUNT];

//Indicator array
float fIndVals[NUM_STOCKS][NUM_FLOAT_INDS][COUNT];
char bIndVals[NUM_STOCKS][NUM_BOOL_INDS][COUNT];

//Used to compare GPU results to sequential results if desired
float * roia;

//USed to compare signals generated. Not used anymore.
char * signalsa;

//Builds the function sets for lilgp
int app_build_function_sets ( void )
{
     function_set fset;
     user_treeinfo tree_map[2];

     function sets[36] =
     {
	   { f_AND,NULL,NULL,2,AND,FUNC_DATA,-1,0,BOOLEAN,{BOOLEAN, BOOLEAN} },
	   { f_OR,NULL,NULL,2,OR,FUNC_DATA,-1,0,BOOLEAN,{BOOLEAN, BOOLEAN} },
	   { f_NOT,NULL,NULL,1,NOT,FUNC_DATA,-1,0,BOOLEAN,{BOOLEAN, IGNORE} },
	   { f_GT,NULL,NULL,2,GT,FUNC_DATA,-1,0,BOOLEAN,{DOUBLE, DOUBLE} },
	   { f_LT,NULL,NULL,2,LT,FUNC_DATA,-1,0,BOOLEAN,{DOUBLE, DOUBLE} },
	   { f_MA1,NULL,NULL,0,MA1,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_MA5,NULL,NULL,0,MA5,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_MA10,NULL,NULL,0,MA10,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_MA15,NULL,NULL,0,MA15,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_MA25,NULL,NULL,0,MA25,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_MA50,NULL,NULL,0,MA50,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_MA75,NULL,NULL,0,MA75,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_MA100,NULL,NULL,0,MA100,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_MA150,NULL,NULL,0,MA150,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_MA200,NULL,NULL,0,MA200,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_EA25,NULL,NULL,0,EA25,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_EA5,NULL,NULL,0,EA5,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_EA9,NULL,NULL,0,EA9,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_EA15,NULL,NULL,0,EA15,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_EA20,NULL,NULL,0,EA20,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_NVIG,NULL,NULL,0,NVIG,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_NVIL,NULL,NULL,0,NVIL,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_PVIG,NULL,NULL,0,PVIG,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_PVIL,NULL,NULL,0,PVIL,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_CLOSING,NULL,NULL,0,CLOSING,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_TYPICAL,NULL,NULL,0,TYPICAL,TERM_NORM,-1,0,DOUBLE,{IGNORE, IGNORE} },
	   { f_MACDL,NULL,NULL,0,MACDL,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_MACDG,NULL,NULL,0,MACDG,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_CCIG,NULL,NULL,0,CCIG,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_CCIL,NULL,NULL,0,CCIL,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_EOMG,NULL,NULL,0,EOMG,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_EOML,NULL,NULL,0,EOML,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_MFIG,NULL,NULL,0,MFIG,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_MFIL,NULL,NULL,0,MFIL,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_MACDGZ,NULL,NULL,0,MACDGZ,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} },
	   { f_MACDLZ,NULL,NULL,0,MACDLZ,TERM_NORM,-1,0,BOOLEAN,{IGNORE, IGNORE} }};
     
     binary_parameter ( "app.use_ercs", 0 );
     if ( atoi ( get_parameter ( "app.use_ercs" ) ) )
          fset.size = 36;
     else
          fset.size = 36;
     fset.cset = sets;

     tree_map[0].fset = 0;
     tree_map[0].return_type = BOOLEAN;
     tree_map[0].name = "TREE1";
     tree_map[1].fset = 0;
     tree_map[1].return_type = BOOLEAN;
     tree_map[1].name = "TREE2";



     return function_sets_init ( &fset, 1, &tree_map, 2 );
}

//Evaluate fitness sequentially (standard way of evaluating in lilgp)
void app_eval_fitness_seq ( individual *ind, unsigned long int indNum )
{
     int i, j;
     int signal;
     float profit;
     float money;
     unsigned long int numShares;
     globaldata* g = get_globaldata();
     set_current_individual ( ind );


     ind->r_fitness = 0.0;
     ind->hits = 0;
     profit = 0.0f;

     //For every stock
     for ( i = 0; i < NUM_STOCKS; i++ ){
    	 //Set money to initial amount
    	 money = (float)INIT_MONEY;
    	 g->stockNum = i;

    	 //For all training days
    	 for(j = START_DAY; j <= END_DAY; j++){
			  signal = 0;
			  g->day = j;

			  //Calculate the signal
			  signal += evaluate_tree ( ind->tr[0].data, 0 );
			  signal -= evaluate_tree ( ind->tr[1].data, 1 );

			  //Same logic as in GPU
			  if(signal > 0 && numShares <= 0){
				  money = money - COMMISION_FEE;
				  numShares = floor(money / fIndVals[i][vCP][j]);
				  money = money - (numShares * fIndVals[i][vCP][j]);
				}else if (signal < 0 && numShares > 0){
					money = money + (numShares * fIndVals[i][vCP][j]);
					money = money - COMMISION_FEE;
					numShares = 0;
				}
    	 }

    	 if(numShares > 0){
    		money = money + (numShares * fIndVals[i][vCP][END_DAY]);
    		money = money - COMMISION_FEE;
			numShares = 0;
    	 }
    	 profit += (money - INIT_MONEY);
     }

     float roi = profit / (INIT_MONEY * NUM_STOCKS);
     if(COMPARE_TO_SEQ){
    	 roia[indNum] = roi;
     }

     //Set fitness
     ind->r_fitness = roi;
     ind->s_fitness = 100 - ind->r_fitness;
     ind->a_fitness = 1/(1+ind->s_fitness);
     ind->evald = EVAL_CACHE_VALID;
}


//Evaluates a population
void app_eval_pop(population * pop){
	//If comparing to sequential evaluation, allocate arrays to store information to compare
	if(COMPARE_TO_SEQ){
		signalsa = malloc((POP_SIZE * NUM_STOCKS * TRAINING_DAYS * sizeof(char)));
		roia = malloc(sizeof(float) * POP_SIZE);
	}

	//If using GPU, call the funciton to evaluate on GPU
	if(USE_GPU){
		app_eval_pop_gpu(pop);
	}

	int i;

	//If using Sequential, call eval function for each invalid individual
	if(USE_SEQ){
		for ( i = 0; i < pop->size; ++i ){
			if ( pop->ind[i].evald != EVAL_CACHE_VALID ){
			app_eval_fitness_seq ( (pop->ind)+i, i);
			}
		}
	}

	//If comparing to sequential, go through and compare
	if(COMPARE_TO_SEQ){
		int curIndCount = 0;
		for ( i = 0; i < pop->size; ++i ){
			if ( pop->ind[i].evald != EVAL_CACHE_VALID ){
				app_eval_fitness2 ( (pop->ind)+i, curIndCount);
				curIndCount++;
			}
		}
	}

	if(COMPARE_TO_SEQ){
		free(signalsa);
		free(roia);
	}
}

//Evaluates the population on the GPU
void app_eval_pop_gpu(population *pop){

	int i;
	int j;
	unsigned long int totalSize = 0;
	unsigned long int countInds = 0;

	//Go through population and count size/number of individuals that need evaluated (do not have their fitness already cached)
	for(i = 0; i < pop->size; i++){
		if(((pop->ind)+i)->evald != EVAL_CACHE_VALID){
			for(j = 0; j < tree_count; j++){
				totalSize += (((pop->ind)+i)->tr[j].size) + 1;
			}
			countInds++;
		}
	}

	//Array for storing starting locations of all individuals/trees
	unsigned long int startInds[countInds * tree_count];
	//Array of RPN strings for all individuals/trees
	char *indstrings;
	indstrings = malloc(sizeof(char)*(totalSize));
	 if(indstrings == NULL){
		 printf("MALLOC ERROR - indstrings = malloc(sizeof(char *)*(pop->size * tree_count))");
		 exit(1);
	 }

	 //For each individual that needs evaluated, print that individuals trees in RPN to array, store starting position
	int k;
	int myIndex = 0;
	int * index = &myIndex;
	unsigned long int curIndCount = 0;
	for(i = 0; i < pop->size; i++){
		if(((pop->ind)+i)->evald != EVAL_CACHE_VALID){
			for(j = 0; j < tree_count; j++){
				startInds[(curIndCount * tree_count) + j] = *index;
				print_individual_rpn_string ( (pop->ind)+i, indstrings, j, index);
			}
			curIndCount++;
		}
	}

	//Allocate space for ROI results
	float * roi = malloc(sizeof(float) * countInds);

	if(roi == NULL){
		printf("MALLOC ERROR ROI");
		exit(1);
	}

	//Call method to evaluate on GPU (from gp_cuda.cu)
	eval_pop_gpu(indstrings, totalSize, (countInds * tree_count), signalsa, countInds, roi, startInds, COMPARE_TO_SEQ);


	//Iterate through again, setting ROI for all individuals
	curIndCount = 0;
	for(i = 0; i < pop->size; i++){
		if(((pop->ind)+i)->evald != EVAL_CACHE_VALID){
			pop->ind[i].r_fitness = roi[curIndCount];

			if(COMPARE_TO_SEQ){
				roia[curIndCount] = roi[curIndCount];
			}

			pop->ind[i].hits = 0;

			double s_fit = 100 - pop->ind[i].r_fitness;
			if(s_fit < 0)s_fit = 0;

			pop->ind[i].s_fitness = s_fit;
			pop->ind[i].a_fitness = 1/(1+pop->ind[i].s_fitness);
			if(!COMPARE_TO_SEQ){
				pop->ind[i].evald = EVAL_CACHE_VALID;
			}
			curIndCount++;
		}
	}

	//Free the arrays we don't need anymore
	free(roi);
	free(indstrings);
}

//Copies indicator values to GPU (will also allocate other necessary arrays)
void init_copy_data_gpu(){
	//Allocate arrays to copy
	float * fInds = malloc(sizeof(float) * (NUM_STOCKS * NUM_FLOAT_INDS * TRAINING_DAYS));
	char * bInds = malloc(sizeof(char) * (NUM_STOCKS * NUM_BOOL_INDS * TRAINING_DAYS));

	if(fInds == NULL || bInds == NULL){// || days == NULL){
		printf("MALLOC ERROR - INIT_COPY_DATA_GPU");
		exit(1);
	}

	int i;
	int j;
	int k;

	//Multidimensional array is used in host memory, flattened array is used on GPU
	//Need to copy values from multidimensional to proper places in single dimension array
	//This will result in GPU array being indexed by day/indicator/stock for better coalescence
	for(i = 0; i < NUM_STOCKS; i++){
		for(j = 0; j < NUM_FLOAT_INDS; j++){
			for(k = START_DAY; k <= END_DAY; k++){
				fInds[((k-START_DAY) * (NUM_FLOAT_INDS * NUM_STOCKS)) + (j * NUM_STOCKS) + i] = fIndVals[i][j][k];
			}
		}
	}

	for(i = 0; i < NUM_STOCKS; i++){
		for(j = 0; j < NUM_BOOL_INDS; j++){
			for(k = START_DAY; k <= END_DAY; k++){
				bInds[((k-START_DAY) * (NUM_BOOL_INDS * NUM_STOCKS)) + (j * NUM_STOCKS) + i] = bIndVals[i][j][k];
			}
		}
	}

	//Copy this data and allocate arrays on GPU
	copy_init_data(fInds, bInds);

	//Free arrays
	free(fInds);
	free(bInds);
}

//Old fitness evaluation, not needed
void app_eval_fitness ( individual *ind){

}

//Sequential stack-based evaluation, to verify GPU version returns same results
void app_eval_fitness2 ( individual *ind, int indNum )
{
	globaldata* g=get_globaldata();
    set_current_individual ( ind );

    //Signal pointer
	int **signals;
    signals = malloc(sizeof(int *)*NUM_STOCKS);
    if(signals == NULL){
    	printf("MALLOC ERROR - signals = malloc(sizeof(int *)*NUM_STOCKS;");
    	exit(1);
    }

    int i;
    int j;
    int k;

     for(i = 0; i < NUM_STOCKS; i++){
    	 signals[i] = malloc(TRAINING_DAYS * sizeof(int));
    	 if(signals[i] == NULL){
    	   	 printf("MALLOC ERROR - signals[i] = malloc(TRAINING_DAYS * sizeof(int));");
    	  	 exit(1);
    	 }

    	 for(j = 0; j < TRAINING_DAYS; j++){
    		 signals[i][j] = 0;
    	 }
     }

     //Convert each individual to RPN and generate signals for that individual
     int x = 0;
     int *xP = &x;
     for(i = 0; i < tree_count; i++){
    	 *xP = 0;
    	 char indstring[ind->tr[i].size + 1];
    	 for(j = 0; j < ind->tr[i].size + 1; j++){
    		 indstring[i] = 0;
    	 }
    	 print_individual_rpn_string ( ind, &indstring, i, xP);
    	 for(j = 0; j < NUM_STOCKS; j++){
			 generateSignals(indstring, ind->tr[i].size, j, i, signals, START_DAY, END_DAY, indNum);
		}
     }

     //Initialize the values for money/num trades
     float money[NUM_STOCKS];
     int numTrades[NUM_STOCKS];
     for(i = 0; i < NUM_STOCKS; i++){
    	 money[i] = INIT_MONEY;
    	 numTrades[i] = 0;
     }

     //Evalute the signals that were generated
     evalSignals(signals, money, numTrades, START_DAY, END_DAY);


     /* Originally compared signals returned by GPU, but signals are no longer stored in GPU version
	 for(j = 0; j < NUM_STOCKS; j++){
		 for(k = 0; k < TRAINING_DAYS; k++){
			 if(signals[j][k] != signalsa[(indNum*NUM_STOCKS*TRAINING_DAYS) + (j * TRAINING_DAYS) + k]){
		//	 printf("SIGNAL #%i,%i,%i: %f, %f\n", indNum, j, k, signals[j][k], signalsa[(indNum*NUM_STOCKS*TRAINING_DAYS) + (j * TRAINING_DAYS) + k]);
			 }
		 }
	 }*/

     //Calculate the profit
     float profit = 0;
	  for(i = 0; i < NUM_STOCKS; i++){
			  profit += money[i] - INIT_MONEY;
	  }

	  //ROI Calculation
	  float roi = profit / (INIT_MONEY * NUM_STOCKS);

	  //If comparing, check to see that ROI is same as GPU ROI
	  if(roi != roia[indNum]){
		  printf("ROI #%i: %f, %f\n", indNum, roi, roia[indNum]);
		  if((fabs(roi - roia[indNum])) > maxDiff){
			  maxDiff = (fabs(roi - roia[indNum]));

		  }
		  printf("MAXDIF: %f\n", maxDiff);
	  }

	  //Set fitness of the individual
	 ind->r_fitness = roi;
	 ind->hits = 0;
	 double s_fit = 20 - ind->r_fitness;
	 if(s_fit < 0)s_fit = 0;
     ind->s_fitness = s_fit;
     ind->a_fitness = 1/(1+ind->s_fitness);
     ind->evald = EVAL_CACHE_VALID;
     for(i = 0; i < NUM_STOCKS; i++){
		 free(signals[i]);
		 }
		free(signals);

}

//Evaluates the best individual on training data, this is just used for outputting results and comparing later on
void evalBestTrain(individual * ind){
		int **signals;
			 signals = malloc(sizeof(int *)*NUM_STOCKS);
			 if(signals == NULL){
				 printf("MALLOC ERROR - signals = malloc(sizeof(int *)*NUM_STOCKS;");
				 exit(1);
			 }
			 int i;
			 int j;
			 int k;

			 for(i = 0; i < NUM_STOCKS; i++){
				 signals[i] = malloc(TRAINING_DAYS * sizeof(int));
				 if(signals == NULL){
					 printf("MALLOC ERROR - signals[i] = malloc(TRAINING_DAYS * sizeof(int));");
					 exit(1);
				 }

		    	 for(j = 0; j < TRAINING_DAYS; j++){
		    		 signals[i][j] = 0;
		    	 }
			 }

				 int x = 0;
				  int *xP = &x;
				  for(i = 0; i < tree_count; i++){
					  *xP = 0;
					 char indstring[ind->tr[i].size + 1];
					 print_individual_rpn_string ( ind, &indstring, i, xP);
					 //printf("TESTING TREE %i: %s\n\n", i, indstring);
					 for(j = 0; j < NUM_STOCKS; j++){
						 generateSignals(indstring, ind->tr[i].size, j, i, signals, START_DAY, END_DAY, 0);
					 }
				  }

				  float money[NUM_STOCKS];
				  int numTrades[NUM_STOCKS];
				     for(i = 0; i < NUM_STOCKS; i++){
				    	 money[i] = INIT_MONEY;
				    	 numTrades[i] = 0;
				     }

				     char myFName[50];
				     sprintf(myFName, "BESTINDTRAINING.%s.%i.csv", TEST_NAME, testRunNum);
				  float myROI = evalSignalsOutput(signals, money, numTrades, START_DAY, END_DAY, myFName);

				  fprintf(myOutputFile, "ROI on Training Cases:, %f\n", myROI);
				  fprintf(myOutputFile, "ROI/100Days on Training Cases:, %f\n", (100 * myROI) / TRAINING_DAYS);

				  		for(i = 0; i < NUM_STOCKS; i++){
						 free(signals[i]);
						 }
				  		free(signals);


}

//Evaluates the best individual on testing data, this is just used for outputting results and comparing later on
void evalBestTest(individual * ind){


		FILE * fp = fopen("BESTINDTEST.csv", "w");

		int **signals;
		 signals = malloc(sizeof(int *)*NUM_STOCKS);
		 if(signals == NULL){
			 printf("MALLOC ERROR - signals = malloc(sizeof(int *)*NUM_STOCKS;");
			 exit(1);
		 }
		 int i;
		 int j;
		 int k;

		 for(i = 0; i < NUM_STOCKS; i++){
			 signals[i] = malloc(TESTING_DAYS * sizeof(int));
			 if(signals == NULL){
				 printf("MALLOC ERROR - signals[i] = malloc(TESTING_DAYS * sizeof(int));");
				 exit(1);
			 }

	    	 for(j = 0; j < TESTING_DAYS; j++){
	    		 signals[i][j] = 0;
	    	 }
		 }


		  int x = 0;
		  int *xP = &x;
		  for(i = 0; i < tree_count; i++){
			  *xP = 0;
			 char indstring[ind->tr[i].size + 1];
			 print_individual_rpn_string ( ind, &indstring, i, xP);
			 //printf("TESTING TREE %i: %s\n\n", i, indstring);
			 for(j = 0; j < NUM_STOCKS; j++){
				 generateSignals(indstring, ind->tr[i].size, j, i, signals, START_TEST_DAY, END_TEST_DAY, 0);
			 }
		  }


		  float money[NUM_STOCKS];
		  int numTrades[NUM_STOCKS];
		     for(i = 0; i < NUM_STOCKS; i++){
		    	 money[i] = INIT_MONEY;
		    	 numTrades[i] = 0;
		     }


		 char myFName[50];
		 sprintf(myFName, "BESTINDTESTING.%s.%i.csv", TEST_NAME, testRunNum);
		 float myROI = evalSignalsOutput(signals, money, numTrades, START_TEST_DAY, END_TEST_DAY, myFName);
		 fprintf(myOutputFile, "ROI on Test Cases:, %f\n", myROI);
		 fprintf(myOutputFile, "ROI/100Days on Test Cases:, %f\n", (100 * myROI) / TESTING_DAYS);



		 for(i = 0; i < NUM_STOCKS; i++){
		 free(signals[i]);
		 }
		free(signals);

}

//Evaluates signals and outputs results. This allows you to see trades made and profit of all individual stocks
//Used in combination with evaluateBestTest and evaluateBestTrain to print out results to file
float evalSignalsOutput(int **signals, float money[], int numTrades[], int startDay, int endDay, char* file){
	int i;
	int stocks;
	int numShares = 0;
	FILE * fp = fopen(file, "w");

	fprintf(fp, "BEST IND SIGNALS:\n\n");
	for(stocks = 0; stocks < NUM_STOCKS; stocks++){
		fprintf(fp, "STOCK: %s\n", STOCK_NAMES[stocks]);
		for(i = startDay; i <= endDay; i++){
			if (signals[stocks][i-startDay] > 0 && numShares <= 0){
				money[stocks] = money[stocks] - COMMISION_FEE;
				numShares = floor(money[stocks] / stockDays[stocks][i].close);
				money[stocks] = money[stocks] - (numShares * stockDays[stocks][i].close);
				numTrades[stocks] = numTrades[stocks] + 1;
				fprintf(fp, "%s, BUY, %i@, %f, %f\n", stockDays[stocks][i].date, numShares, stockDays[stocks][i].close, money[stocks]);
			}else if (signals[stocks][i-startDay] < 0 && numShares > 0){
				money[stocks] = money[stocks] + (numShares * stockDays[stocks][i].close);
				money[stocks] = money[stocks] - COMMISION_FEE;
				numTrades[stocks] = numTrades[stocks] + 1;
				fprintf(fp, "%s, SELL, %i@, %f, %f\n", stockDays[stocks][i].date, numShares, stockDays[stocks][i].close, money[stocks]);
				numShares = 0;
			}
		}

		if (numShares > 0){
			money[stocks] = money[stocks] + (numShares * stockDays[stocks][endDay].close);
			//money[stocks] = money[stocks] - COMMISION_FEE;
			fprintf(fp, "%s, SELL, %i@, %f, %f\n", stockDays[stocks][endDay].date, numShares, stockDays[stocks][endDay].close, money[stocks]);
			numShares = 0;
			numTrades[stocks] = numTrades[stocks] + 1;
		}
	}

	  float profit = 0.0f;
	  int totalTrades = 0;
	  for(i = 0; i < NUM_STOCKS; i++){
			  profit += (money[i] - INIT_MONEY);
			  fprintf(fp, "PROFIT ON %s: %f\n", STOCK_NAMES[i], (money[i] - INIT_MONEY));
			  fprintf(fp, "ROI ON %s: %f\n", STOCK_NAMES[i], ((money[i] - INIT_MONEY) / INIT_MONEY));
			  fprintf(fp, "NUM TRADES ON %s: %i\n", STOCK_NAMES[i], numTrades[i]);
			  totalTrades += numTrades[i];
	  }

	  float roi = profit / (INIT_MONEY * NUM_STOCKS);

	  fprintf(fp, "TOTAL PROFIT: %f\n\n", profit);
	  fprintf(fp, "TOTAL ROI: %f\n\n", roi);
	  fprintf(fp, "#TRADES: %i\n\n", totalTrades);
	  fprintf(fp, "SIGNALS: \n \n");
	  //printf("SIGNALS:");
	  int j;
	  for(i = 0; i < NUM_STOCKS; i++){
		   for(j = startDay; j <= endDay; j++){
			  //printf("DAY: %i", j);
			  fprintf(fp, "%i, ", signals[i][j-startDay]);
		  }
		  fprintf(fp, "\n");
	  }
	  fclose(fp);

	  return roi;
}

//Evaluates the signals from the sequential stack based interpreter
void evalSignals(int **signals, float money[], int numTrades[], int startDay, int endDay){
	int i;
	int stocks;
	int j;
	int numShares = 0;

	for(stocks = 0; stocks < NUM_STOCKS; stocks++){
		for(i = startDay; i <= endDay; i++){
			if (signals[stocks][i-startDay] > 0 && numShares <= 0){
				money[stocks] = money[stocks] - COMMISION_FEE;
				numShares = floor(money[stocks] / fIndVals[stocks][vCP][i]);
				money[stocks] = money[stocks] - (numShares * fIndVals[stocks][vCP][i]);
				numTrades[stocks] = numTrades[stocks] + 1;
			}else if (signals[stocks][i-startDay] < 0 && numShares > 0){
				money[stocks] = money[stocks] + (numShares * fIndVals[stocks][vCP][i]);
				money[stocks] = money[stocks] - COMMISION_FEE;
				numShares = 0;
				numTrades[stocks] = numTrades[stocks] + 1;
			}
		}

		if (numShares > 0){
			money[stocks] = money[stocks] + (numShares * fIndVals[stocks][vCP][endDay]);
			numShares = 0;
			numTrades[stocks] = numTrades[stocks] + 1;
		}
	}
}

//Sequential stack based interpreter
#define push(x) {stack[sp] = x; sp++;}
void generateSignals(char *rpn, int len, int stockNum, int treeNum, int **signals, int startDay, int endDay, int indNum){
	float stack[20];
	int sp = 0;
	int i;
	int j;
	int k;

	int day;

	for(day = startDay; day <= endDay; day++){
		int sp = 0;
		for(i = 0; i < len; i++){
			char op = rpn[i];
			float sp1;
			float sp2;

			if(op == 0){
				break;
			}

			if(sp >= 17){
				printf("STACK OVERFLOW");
				exit(1);
			}

			if(op == AND[0]){
				sp1 = stack[sp-1];
				sp2 = stack[sp-2];
				sp -= 2;
				push(sp1 && sp2);
				//printf("AND(%f,%f) -> %f\n", sp1, sp2, stack[sp-1]);
			}else if(op == OR[0]){
				sp1 = stack[sp-1];
				sp2 = stack[sp-2];
				sp -= 2;
				push(sp1 || sp2);
				//printf("OR(%f,%f) -> %f\n", sp1, sp2, stack[sp-1]);
			}else if(op == NOT[0]){
				sp1 = stack[sp-1];
				sp -= 1;
				push(!sp1);
				//printf("NOT(%f) -> %f\n", sp1, stack[sp-1]);
			}else if(op == GT[0]){
				sp1 = stack[sp-2];
				sp2 = stack[sp-1];
				sp -= 2;
				push(sp1 > sp2);
				//printf(">(%f,%f) -> %f\n", sp1, sp2, stack[sp-1]);
			}else if(op == LT[0]){
				sp1 = stack[sp-2];
				sp2 = stack[sp-1];
				sp -= 2;
				push(sp1 < sp2);
				//printf("<(%f,%f) -> %f\n", sp1, sp2, stack[sp-1]);
			}else if(op == MA1[0]){
				push(fIndVals[stockNum][vMA1][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vMA1][day], stack[sp-1]);
			}else if(op == MA5[0]){
				push(fIndVals[stockNum][vMA5][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vMA5][day], stack[sp-1]);
			}else if(op == MA10[0]){
				push(fIndVals[stockNum][vMA10][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vMA10][day], stack[sp-1]);
			}else if(op == MA15[0]){
				push(fIndVals[stockNum][vMA15][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vMA15][day], stack[sp-1]);
			}else if(op == MA25[0]){
				push(fIndVals[stockNum][vMA25][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vMA25][day], stack[sp-1]);
			}else if(op == MA50[0]){
				push(fIndVals[stockNum][vMA50][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vMA50][day], stack[sp-1]);
			}else if(op == MA75[0]){
				push(fIndVals[stockNum][vMA75][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vMA75][day], stack[sp-1]);
			}else if(op == MA100[0]){
				push(fIndVals[stockNum][vMA100][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vMA100][day], stack[sp-1]);
			}else if(op == MA150[0]){
				push(fIndVals[stockNum][vMA150][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vMA150][day], stack[sp-1]);
			}else if(op == MA200[0]){
				push(fIndVals[stockNum][vMA200][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vMA200][day], stack[sp-1]);
			}else if(op == EA5[0]){
				push(fIndVals[stockNum][vEMA5][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vEMA5][day], stack[sp-1]);
			}else if(op == EA9[0]){
				push(fIndVals[stockNum][vEMA9][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vEMA9][day], stack[sp-1]);
			}else if(op == EA15[0]){
				push(fIndVals[stockNum][vEMA15][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vEMA15][day], stack[sp-1]);
			}else if(op == EA20[0]){
				push(fIndVals[stockNum][vEMA20][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vEMA20][day], stack[sp-1]);
			}else if(op == EA25[0]){
				push(fIndVals[stockNum][vEMA25][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vEMA25][day], stack[sp-1]);
			}else if(op == CLOSING[0]){
				push(fIndVals[stockNum][vCP][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vCP][day], stack[sp-1]);
			}else if(op == TYPICAL[0]){
				push(fIndVals[stockNum][vTP][day]);
				//printf("PUSH(%f) -> %f\n", fIndVals[stockNum][vTP][day], stack[sp-1]);
			}else if(op == NVIG[0]){
				push((float)bIndVals[stockNum][vNVIG][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vNVIG][day], stack[sp-1]);
			}else if(op == NVIL[0]){
				push((float)bIndVals[stockNum][vNVIL][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vNVIL][day], stack[sp-1]);
			}else if(op == PVIG[0]){
				push((float)bIndVals[stockNum][vPVIG][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vPVIG][day], stack[sp-1]);
			}else if(op == PVIL[0]){
				push((float)bIndVals[stockNum][vPVIL][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vPVIL][day], stack[sp-1]);
			}else if(op == MACDL[0]){
				push((float)bIndVals[stockNum][vMACDL][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vMACDL][day], stack[sp-1]);
			}else if(op == MACDG[0]){
				push((float)bIndVals[stockNum][vMACDG][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vMACDG][day], stack[sp-1]);
			}else if(op == MACDGZ[0]){
				push((float)bIndVals[stockNum][vMACDGZ][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vMACDGZ][day], stack[sp-1]);
			}else if(op == MACDLZ[0]){
				push((float)bIndVals[stockNum][vMACDLZ][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vMACDLZ][day], stack[sp-1]);
			}else if(op == CCIG[0]){
				push((float)bIndVals[stockNum][vCCIG][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vCCIG][day], stack[sp-1]);
			}else if(op == CCIL[0]){
				push((float)bIndVals[stockNum][vCCIL][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vCCIL][day], stack[sp-1]);
			}else if(op == EOML[0]){
				push((float)bIndVals[stockNum][vEOML][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vEOML][day], stack[sp-1]);
			}else if(op == EOMG[0]){
				push((float)bIndVals[stockNum][vEOMG][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vEOMG][day], stack[sp-1]);
			}else if(op == MFIG[0]){
				push((float)bIndVals[stockNum][vMFIG][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vMFIG][day], stack[sp-1]);
			}else if(op == MFIL[0]){
				push((float)bIndVals[stockNum][vMFIL][day]);
				//printf("PUSH(%f) -> %f\n", (float)bIndVals[stockNum][vMFIL][day], stack[sp-1]);
			}else{
				printf("ERROR: UNKNOWN OPERAND");
				exit(1);
				break;
			}
		}

		signals[stockNum][day-startDay] += (char)((((-treeNum)*2)+1)*(char)stack[0]);
	}
}

//Called at end of evaluation, prints out best individuals results
int app_end_of_evaluation ( int gen, multipop *mpop, int newbest,
                           popstats *gen_stats, popstats *run_stats )
{
     int i;
     DATATYPE v;
     globaldata* g=get_globaldata();

     if ( newbest )
     {
          output_stream_open ( OUT_USER );
          
          for ( i = -100; i <= 100; ++i )
          {
               //g->x = (float)i*.01;
               //v = evaluate_tree ( run_stats[0].best[0]->ind->tr[0].data, 0 );
               //oprintf ( OUT_USER, 50, "%lf %lf\n", 0, v );
          }
          
          output_stream_close ( OUT_USER );
          
          if ( run_stats[0].best[0]->ind->hits == fitness_cases )
               return 1;
        evalBestTrain(run_stats[0].best[0]->ind);
          evalBestTest(run_stats[0].best[0]->ind);
     }
     return 0;
}

void app_end_of_breeding ( int gen, multipop *mpop )
{
     return;
}

int app_create_output_streams ( void )
{
     if ( create_output_stream ( OUT_USER, ".fn", 1, "w", 0 ) !=
         OUTPUT_OK )
          return 1;

     return 0;
}

int app_initialize ( int startfromcheckpoint )
{
     int i;
     double x, y;
     char *param;

     if ( !startfromcheckpoint )
     {
	  oprintf ( OUT_PRG, 50, "not starting from checkpoint file.\n" );
	  
	  param = get_parameter ( "app.fitness_cases" );
	  if ( param == NULL )
	       fitness_cases = 200;
	  else
	  {
	       fitness_cases = atoi ( param );
	       if ( fitness_cases < 0 )
		    error ( E_FATAL_ERROR, "invalid value for \"app.fitness_cases\"." );
	  }
	  
	  app_fitness_cases[0] = (double *)MALLOC ( fitness_cases *
						   sizeof ( double ) );
	  app_fitness_cases[1] = (double *)MALLOC ( fitness_cases *
						   sizeof ( double ) );
	  
	  oprintf ( OUT_PRG, 50, "%d fitness cases:\n", fitness_cases );
	  for ( i = 0; i < fitness_cases; ++i )
	  {
	       x = (random_double()*2.0)-1.0;
	       
	       /* change this line to modify the goal function. */
	       y = x*x*x*x + x*x*x + x*x + x;
	       
	       app_fitness_cases[0][i] = x;
	       app_fitness_cases[1][i] = y;
     
	       oprintf ( OUT_PRG, 50,
			"    x = %12.5lf, y = %12.5lf\n", x, y );
	  }
     }
     else
     {
	  oprintf ( OUT_PRG, 50, "started from checkpoint file.\n" );
     }

     param = get_parameter ( "app.value_cutoff" );
     if ( param == NULL )
          value_cutoff = 1.e15;
     else
          value_cutoff = strtod ( param, NULL );
     

     for(i=0; i < NUM_STOCKS; i++){
     	printf("%i: %s\n", i, STOCK_NAMES[i]);
     	initStocks(i);
     }

     //Copies the indicator values to files
     printf("STARTING OUTPUT OF INDICATORS\n");
     for(i = 0; i < NUM_STOCKS; i++){
    	 printf("STARTING STOCK: %i\n", i);
    	 char filename[50];
    	 sprintf(filename, "Indicators/%s.csv", STOCK_NAMES[i]);
    	 FILE * fp = fopen(filename, "w");
	
    	 fprintf(fp, "Day#, MA1 a, MA5 b, MA10 c, MA15 d, MA25 e, MA50 f, MA75 g, MA100 h, MA150 i, MA200 j, EMA5 l, EMA9 m, EMA15 n, EMA20 o, EMA25 p, CP u, TP v, NVIG q, NVIL r, PVIG s, PVIL t, MACDG x, MACDL w, MACDGZ K, MACDLZ L, CCIG y, CCIL z, EOMG A, EOML B, MFIG E, MFIL F, END\n");

    	 int k;
    	 int j;
    	 //printf("STARTING COUNT LOOP");
    	 for(k = START_DAY; k <= END_DAY; k++){
    		 fprintf(fp, "%i, ", k);

    		 //printf("STARTING FLOAT INDS");
    		 for(j = 0; j < NUM_FLOAT_INDS; j++){
    			fprintf(fp, "%f, ", fIndVals[i][j][k]);
    		 }

    		 //printf("STARTING BOOL INDS");
    		 for(j = 0; j < NUM_BOOL_INDS; j++){
				 fprintf(fp, "%f, ", (float)bIndVals[i][j][k]);
			 }

    		 fprintf(fp, "END\n");
    	 }
    	 fclose(fp);
     }
     printf("ABOUT TO COPY INIT DATA TO GPU");
     fflush(stdout);
     init_copy_data_gpu();
     return 0;
}

void app_uninitialize ( void )
{
     FREE ( app_fitness_cases[0] );
     FREE ( app_fitness_cases[1] );
     uninit_data();
}

void app_write_checkpoint ( FILE *f )
{
     int i;
     fprintf ( f, "fitness-cases: %d\n", fitness_cases );
     for ( i = 0; i < fitness_cases; ++i )
     {
	  write_hex_block ( app_fitness_cases[0]+i, sizeof(double), f );
	  fputc ( ' ', f );
	  write_hex_block ( app_fitness_cases[1]+i, sizeof(double), f );
	  fprintf ( f, " %.5lf %.5lf\n",
		   app_fitness_cases[0][i], app_fitness_cases[1][i] );
     }
}

void app_read_checkpoint ( FILE *f )
{
     int i;
     
     fscanf ( f, "%*s %d\n", &fitness_cases );
     
     app_fitness_cases[0] = (double *)MALLOC ( fitness_cases *
                                              sizeof ( double ) );
     app_fitness_cases[1] = (double *)MALLOC ( fitness_cases *
                                              sizeof ( double ) );

     for ( i = 0; i < fitness_cases; ++i )
     {
	  read_hex_block ( app_fitness_cases[0]+i, sizeof ( double ), f );
	  fgetc ( f );
	  read_hex_block ( app_fitness_cases[1]+i, sizeof ( double ), f );
	  fscanf ( f, " %*f %*f\n" );
	  fprintf ( stderr, "%.5lf %.5lf\n", app_fitness_cases[0][i],
		   app_fitness_cases[1][i] );
     }
}



//Initializes stock data, reading in from file and calculating indicators
void initStocks(int stockNum){
	FILE *fp;
		const char *mode = "r";
		const char *file = STOCK_NAMES[stockNum];

		printf("FILE: %s\n", file);
		fp= fopen(file, mode);

		char line[75];
		char dates[COUNT][10];
		float highs[COUNT];
		float lows[COUNT];
		float opens[COUNT];
		float closes[COUNT];
		float volumes[COUNT];


		fscanf(fp, "%s", line);

		int curCount = COUNT;
		while(curCount > 0){
			fscanf(fp, "%s", line);

			if(line[strlen(line)-1] == '0' && line[strlen(line)-2]==','){
				//printf("SKIPPED: %s\n", line);
			}else{
				curCount--;
				//printf("ADDED: %s\n", line);


				char *date = strtok(line, ",");
				char *open = strtok(NULL, ",");
				char *high = strtok(NULL, ",");
				char *low = strtok(NULL, ",");
				char *close = strtok(NULL, ",");
				char *volume = strtok(NULL, ",");

				strcpy(dates[curCount], date);
				opens[curCount] = atof(open);
				closes[curCount] = atof(close);
				highs[curCount] = atof(high);
				lows[curCount] = atof(low);
				volumes[curCount] = strtod(volume, NULL);

				StockDay today = {"", atof(high), atof(low), atof(open), atof(close), atof(volume)};
				strcpy(today.date, date);
				stockDays[stockNum][curCount] = today;
			}
		}

		fclose(fp);

		calculateIndicators(START_DAY, END_TEST_DAY, SEED_DAYS, stockNum);
}

void calculateIndicators(int startDay, int endDay, int seedDays, int stockNum){
	int i;

	for(i = 1; i < seedDays; i++){
		//Closing and Typical Price
		fIndVals[stockNum][vCP][0] = stockDays[stockNum][0].close;
		fIndVals[stockNum][vTP][0] = typicalPrice(stockDays[stockNum][0]);
		fIndVals[stockNum][vCP][i] = stockDays[stockNum][i].close;
		fIndVals[stockNum][vTP][i] = typicalPrice(stockDays[stockNum][i]);
	}

	for(i = startDay; i <= endDay; i++){
		//Closing/Typical Price
		fIndVals[stockNum][vCP][i] = stockDays[stockNum][i].close;
		fIndVals[stockNum][vTP][i] = typicalPrice(stockDays[stockNum][i]);
	}

	printf("InitMAs\n");
	initMAs(startDay, endDay, seedDays, stockNum);
	printf("InitPVINVI\n");
	initPVINVI(startDay, endDay, seedDays, stockNum);
	printf("InitMACD\n");
	initMACD(startDay, endDay, seedDays, stockNum);
	printf("InitCCI\n");
	initCCI(startDay, endDay, seedDays, stockNum);
	printf("InitEOM\n");
	initEOM(startDay, endDay, seedDays, stockNum);
	printf("InitMFI\n");
	initMFI(startDay, endDay, seedDays, stockNum);
	printf("ALL DONE\n");
}

void initMFI(int startDay, int endDay, int seedDays, int stockNum){
	float MFI[COUNT];
	float PMF[COUNT];
	float NMF[COUNT];

	int i;
	float tpt;
	float tpy;
	float mf;
	StockDay today;
	StockDay yesterday;

	for(i = 1; i < seedDays; i++){
		today = stockDays[stockNum][i];
		tpt = typicalPrice(today);
		yesterday = stockDays[stockNum][i-1];
		tpy = typicalPrice(yesterday);

		mf = tpt * today.volume;

		if(tpt > tpy){
			PMF[i] = mf;
			NMF[i] = 0;
		}else{
			PMF[i] = 0;
			NMF[i] = mf;
		}

		if( i > 14){
			int j;
			float pmf;
			float nmf;

			for(j = 0; j < 14; j++){
				pmf += PMF[i-j];
				nmf += NMF[i-j];
			}

			float ratio = pmf / nmf;
			MFI[i] = 100 - (100 / (1 + ratio));
		}
	}

	for(i = startDay; i <= endDay; i++){
		today = stockDays[stockNum][i];
		tpt = typicalPrice(today);
		yesterday = stockDays[stockNum][i-1];
		tpy = typicalPrice(yesterday);

		mf = tpt * today.volume;

		if(tpt > tpy){
			PMF[i] = mf;
			NMF[i] = 0;
		}else{
			PMF[i] = 0;
			NMF[i] = mf;
		}

		int j;
		float pmf = 0.0f;
		float nmf = 0.0f;

		for(j = 0; j < 14; j++){
			pmf += PMF[i-j];
			nmf += NMF[i-j];
		}

		float ratio = 0;

		if(nmf != 0){
			ratio = pmf / nmf;
		}

		MFI[i] = 100 - (100 / (1 + ratio));

		float mfiavg = 0.0f;
		for(j = 0; j < 14; j++){
			mfiavg += MFI[i-j];
		}

		bIndVals[stockNum][vMFIG][i] = ((mfiavg / 14) > 80);
		bIndVals[stockNum][vMFIL][i] = ((mfiavg / 14) < 20);

		/*printDay(stockDays[stockNum][i]);
		printf("Day: %i MFI: %f\n", i, MFI[i]);
		printf("Day: %i MFIAvg: %f\n", i, mfiavg/14);
		printf("Day: %i MFIG: %i\n", i, bIndVals[stockNum][vMFIG][i]);
		printf("Day: %i MFIL: %i\n", i, bIndVals[stockNum][vMFIL][i]);*/
	}
}

void initEOM(int startDay, int endDay, int seedDays, int stockNum){
	float EOM[COUNT];
	int i;
	StockDay today;
	StockDay yesterday;

	float MPM;
	float BR;
	for(i = 1; i < seedDays; i++){
		today = stockDays[stockNum][i];
		yesterday = stockDays[stockNum][i-1];
		MPM = ((today.high + today.low) / 2) - ((yesterday.high + yesterday.low) / 2);
		BR = (today.volume / 10000) / (today.high - today.low);
		EOM[i] = MPM / BR;
	}

	int j;
	float temp;
	for(i = startDay; i <= endDay; i++){
		today = stockDays[stockNum][i];
		yesterday = stockDays[stockNum][i-1];
		MPM = ((today.high + today.low) / 2) - ((yesterday.high + yesterday.low) / 2);
		BR = (today.volume / 10000) / (today.high - today.low);
		EOM[i] = MPM / BR;

		temp = 0.0;
		for(j = 0; j < 14; j++){
			temp += EOM[i - j];
		}

		bIndVals[stockNum][vEOMG][i] = ((temp / 14) >= 0);
		bIndVals[stockNum][vEOML][i] = ((temp / 14) < 0);

		/*printDay(stockDays[stockNum][i]);
		printf("Day: %i EOM: %f\n", i, EOM[i]);
		printf("Day: %i Temp: %f\n", i, temp);
		printf("Day: %i EOMAvg: %f\n", i, temp/14);
		printf("Day: %i EOMG: %i\n", i, bIndVals[stockNum][vEOMG][i]);
		printf("Day: %i EOML: %i\n", i, bIndVals[stockNum][vEOML][i]);*/
	}
}

void initCCI(int startDay, int endDay, int seedDays, int stockNum){
	int i;
	float cciToday;

	for(i = startDay; i <= endDay; i++){
		cciToday = calcCCI(i, 14, stockNum);
		bIndVals[stockNum][vCCIG][i] = (cciToday > 100);
		bIndVals[stockNum][vCCIL][i] = (cciToday < -100);

		/*printDay(stockDays[stockNum][i]);
		printf("Day: %i CCI: %f\n", i, cciToday);
		printf("Day: %i CCIG: %i\n", i, bIndVals[stockNum][vCCIG][i]);
		printf("Day: %i CCIL: %i\n", i, bIndVals[stockNum][vCCIL][i]);*/
	}
}

float calcCCI(int day, int numDays, int stockNum){
	float avgtp = 0.0f;
	int i;

	for(i = 0; i < numDays; i++){
		avgtp += fIndVals[stockNum][vTP][day - i];
	}

	avgtp /= numDays;

	float meandev = 0.0f;

	for(i = 0; i < numDays; i++){
		meandev += fabsf(avgtp - fIndVals[stockNum][vTP][day - i]);
	}

	meandev /= numDays;

	return (fIndVals[stockNum][vTP][day] - avgtp) / (0.015 * meandev);
}

void initMACD(int startDay, int endDay, int seedDays, int stockNum){
	int i;

	float MACDLine[COUNT];
	float MACDSig[COUNT];

	MACDLine[0] = 0;
	MACDSig[0] = 0;
	bIndVals[stockNum][vMACDL][0] = 0;
	bIndVals[stockNum][vMACDG][0] = 0;
	bIndVals[stockNum][vMACDGZ][0] = 0;
	bIndVals[stockNum][vMACDLZ][0] = 0;
	float EMA12Day[COUNT];
	float EMA26Day[COUNT];
	EMA12Day[0] = stockDays[stockNum][0].close;
	EMA26Day[0] = stockDays[stockNum][0].close;

	for(i = 1; i < seedDays; i++){
		EMA12Day[i] = (stockDays[stockNum][i].close * 0.153846154f) + (EMA12Day[i-1] * (1 - 0.153846154f));
		EMA26Day[i] = (stockDays[stockNum][i].close * 0.0740740741f) + (EMA26Day[i-1] * (1 - 0.0740740741f));

		MACDLine[i] = EMA12Day[i] - EMA26Day[i];
		MACDSig[i] = (MACDLine[i] * 0.2f) + (MACDLine[i-1] * 0.8f);
		bIndVals[stockNum][vMACDL][i] = (MACDLine[i] < MACDSig[i]);
		bIndVals[stockNum][vMACDG][i] = (MACDLine[i] >= MACDSig[i]);
		bIndVals[stockNum][vMACDGZ][i] = (MACDLine[i] >= 0);
		bIndVals[stockNum][vMACDLZ][i] = (MACDLine[i] < 0);
	}

	for(i = startDay; i <= endDay; i++){
		EMA12Day[i] = (stockDays[stockNum][i].close * 0.153846154f) + (EMA12Day[i-1] * (1 - 0.153846154f));
		EMA26Day[i] = (stockDays[stockNum][i].close * 0.0740740741f) + (EMA26Day[i-1] * (1 - 0.0740740741f));

		MACDLine[i] = EMA12Day[i] - EMA26Day[i];
		MACDSig[i] = (MACDLine[i] * 0.2f) + (MACDLine[i-1] * 0.8f);
		bIndVals[stockNum][vMACDL][i] = (MACDLine[i] < MACDSig[i]);
		bIndVals[stockNum][vMACDG][i] = (MACDLine[i] >= MACDSig[i]);
		bIndVals[stockNum][vMACDGZ][i] = (MACDLine[i] >= 0);
		bIndVals[stockNum][vMACDLZ][i] = (MACDLine[i] < 0);

		/*printDay(stockDays[stockNum][i]);
		printf("Day: %i MACDVal: %f\n", i, MACDLine[i]);
		printf("Day: %i MACDSig: %f\n", i, MACDSig[i]);
		printf("Day: %i MACDL: %i\n", i, bIndVals[stockNum][vMACDL][i]);
		printf("Day: %i MACDG: %i\n", i, bIndVals[stockNum][vMACDG][i]);
		printf("Day: %i MACDGZ: %i\n", i, bIndVals[stockNum][vMACDGZ][i]);
		printf("Day: %i MACDLZ: %i\n", i, bIndVals[stockNum][vMACDLZ][i]);*/


	}
}

void initMAs(int startDay, int endDay, int seedDays, int stockNum){
	int i;

	fIndVals[stockNum][vEMA5][0] = stockDays[stockNum][0].close;
	fIndVals[stockNum][vEMA9][0] = stockDays[stockNum][0].close;
	fIndVals[stockNum][vEMA15][0] = stockDays[stockNum][0].close;
	fIndVals[stockNum][vEMA20][0] = stockDays[stockNum][0].close;
	fIndVals[stockNum][vEMA25][0] = stockDays[stockNum][0].close;

	for(i = 1; i < seedDays; i++){
		//Exponential Moving Average Calculations
		fIndVals[stockNum][vEMA5][i] = EMA(i, 0.05, stockNum, vEMA5);
		fIndVals[stockNum][vEMA9][i] = EMA(i, 0.09, stockNum, vEMA9);
		fIndVals[stockNum][vEMA15][i] = EMA(i, 0.15, stockNum, vEMA15);
		fIndVals[stockNum][vEMA20][i] = EMA(i, 0.20, stockNum, vEMA20);
		fIndVals[stockNum][vEMA25][i] = EMA(i, 0.25, stockNum, vEMA25);
	}

	for(i = startDay; i <= endDay; i++){
		//Simple Moving Average Values
		fIndVals[stockNum][vMA1][i] = MA(i, 1, stockNum);
		fIndVals[stockNum][vMA5][i] = MA(i, 5, stockNum);
		fIndVals[stockNum][vMA10][i] = MA(i, 10, stockNum);
		fIndVals[stockNum][vMA15][i] = MA(i, 15, stockNum);
		fIndVals[stockNum][vMA25][i] = MA(i, 25, stockNum);
		fIndVals[stockNum][vMA50][i] = MA(i, 50, stockNum);
		fIndVals[stockNum][vMA75][i] = MA(i, 75, stockNum);
		fIndVals[stockNum][vMA100][i] = MA(i, 100, stockNum);
		fIndVals[stockNum][vMA150][i] = MA(i, 150, stockNum);
		fIndVals[stockNum][vMA200][i] = MA(i, 200, stockNum);
		//MA1[i] = MA(i, 1, stockNum);
		//MA5[i] = MA(i, 5, stockNum);
		//EMA9[i] = EMA(i, 0.25, stockNum);

		//Exponential Moving Average Values
		fIndVals[stockNum][vEMA5][i] = EMA(i, 0.05, stockNum, vEMA5);
		fIndVals[stockNum][vEMA9][i] = EMA(i, 0.09, stockNum, vEMA9);
		fIndVals[stockNum][vEMA15][i] = EMA(i, 0.15, stockNum, vEMA15);
		fIndVals[stockNum][vEMA20][i] = EMA(i, 0.20, stockNum, vEMA20);
		fIndVals[stockNum][vEMA25][i] = EMA(i, 0.25, stockNum, vEMA25);

		/*printDay(stockDays[stockNum][i]);
		printf("Day: %i MA1: %f\n", i, fIndVals[stockNum][vMA1][i]);
		printf("Day: %i MA5: %f\n", i, fIndVals[stockNum][vMA5][i]);
		printf("Day: %i MA10: %f\n", i, fIndVals[stockNum][vMA10][i]);
		printf("Day: %i MA15: %f\n", i, fIndVals[stockNum][vMA15][i]);
		printf("Day: %i MA25: %f\n", i, fIndVals[stockNum][vMA25][i]);
		printf("Day: %i MA50: %f\n", i, fIndVals[stockNum][vMA50][i]);
		printf("Day: %i MA75: %f\n", i, fIndVals[stockNum][vMA75][i]);
		printf("Day: %i MA100: %f\n", i, fIndVals[stockNum][vMA100][i]);
		printf("Day: %i MA150: %f\n", i, fIndVals[stockNum][vMA150][i]);
		printf("Day: %i MA200: %f\n", i, fIndVals[stockNum][vMA200][i]);

		printf("Day: %i EMA5: %f\n", i, fIndVals[stockNum][vEMA5][i]);
		printf("Day: %i EMA9: %f\n", i, fIndVals[stockNum][vEMA9][i]);
		printf("Day: %i EMA15: %f\n", i, fIndVals[stockNum][vEMA15][i]);
		printf("Day: %i EMA20: %f\n", i, fIndVals[stockNum][vEMA20][i]);
		printf("Day: %i EMA25: %f\n", i, fIndVals[stockNum][vEMA25][i]);*/
	}
}

void initPVINVI(int startDay, int endDay, int seedDays, int stockNum){
	int i;
	float vNVI[COUNT];
	float vPVI[COUNT];
	float NVIYear = 0.0f;
	float PVIYear = 0.0f;

	vNVI[0] = 1;
	vPVI[0] = 1;

	//printf("START START START START");
	StockDay today;
	StockDay yesterday;
	today = stockDays[stockNum][1];
	yesterday = stockDays[stockNum][0];

	for(i = 1; i < seedDays; i++){
		today = stockDays[stockNum][i];
		yesterday = stockDays[stockNum][i-1];
		if(today.volume >= yesterday.volume){
			vNVI[i] = vNVI[i-1];
			vPVI[i] = vPVI[i-1] + (((today.close - yesterday.close) / yesterday.close) * vPVI[i-1]);
		}else{
			vNVI[i] = vNVI[i-1] + (((today.close - yesterday.close) / yesterday.close) * vNVI[i-1]);
			vPVI[i] = vPVI[i-1];
		}

		if(seedDays - i <= 200){
			PVIYear += vPVI[i];
			NVIYear += vNVI[i];
		}
		//printDay(stockDays[stockNum][19]);
	}

	/*printf("PRINT %i\n", 0);
	printDay(stockDays[stockNum][19]);
	printDay(stockDays[stockNum][20]);
	printDay(stockDays[stockNum][21]);*/

	for(i = startDay; i <= endDay; i++){
		today = stockDays[stockNum][i];
		yesterday = stockDays[stockNum][i-1];
		/*printf("PRINT %i\n", i);
		printf("Today: ");
		printDay(today2);
		printf("Yesterday: ");
		printDay(yesterday2);*/

		if(today.volume >= yesterday.volume){
			vNVI[i] = vNVI[i-1];
			vPVI[i] = vPVI[i-1] + (((today.close - yesterday.close) / yesterday.close) * vPVI[i-1]);
		}else{
			vNVI[i] = vNVI[i-1] + (((today.close - yesterday.close) / yesterday.close) * vNVI[i-1]);
			vPVI[i] = vPVI[i-1];
		}

		PVIYear = PVIYear + vPVI[i] - vPVI[i-200];
		NVIYear = NVIYear + vNVI[i] - vNVI[i-200];
		//printf("PVIYear: %f\n", PVIYear);
		//printf("NVIYear: %f\n", NVIYear);

		int j;
		PVIYear = 0;
		NVIYear = 0;
		for (j = 0; j < 200; j++){
			PVIYear += vPVI[i-j];
			NVIYear += vNVI[i-j];
		}

		if(vPVI[i] > (PVIYear / 200)){
			bIndVals[stockNum][vPVIG][i] = 1;
			bIndVals[stockNum][vPVIL][i] = 0;
		}else{
			bIndVals[stockNum][vPVIG][i] = 0;
			bIndVals[stockNum][vPVIL][i] = 1;
		}

		if(vNVI[i] > (NVIYear / 200)){
			bIndVals[stockNum][vNVIG][i] = 1;
			bIndVals[stockNum][vNVIL][i] = 0;
		}else{
			bIndVals[stockNum][vNVIG][i] = 0;
			bIndVals[stockNum][vNVIL][i] = 1;
		}

		/*printDay(stockDays[stockNum][i]);
		printf("Day: %i PVI: %f\n", i, vPVI[i]);
		printf("Day: %i NVI: %f\n", i, vNVI[i]);
		printf("Day: %i PVI/200: %f\n", i, PVIYear / 200);
		printf("Day: %i NVI/200: %f\n", i, NVIYear / 200);
		printf("Day: %i PVIG: %i\n", i, bIndVals[stockNum][vPVIG][i]);
		printf("Day: %i PVIL: %i\n", i, bIndVals[stockNum][vPVIL][i]);
		printf("Day: %i NVIG: %i\n", i, bIndVals[stockNum][vNVIG][i]);
		printf("Day: %i NVIL: %i\n", i, bIndVals[stockNum][vNVIL][i]);*/
	}

	/*printf("PRINT %i\n", 1);
	printDay(stockDays[stockNum][19]);
	printDay(stockDays[stockNum][20]);
	printDay(stockDays[stockNum][21]);*/
}

float typicalPrice(StockDay theDay){
	return (theDay.close + theDay.high + theDay.low) / 3;
}

void printDay(StockDay theDay){
	printf("Date: %s Open: %f High: %f Low: %f Close: %f Volume: %f\n", theDay.date, theDay.open, theDay.high, theDay.low, theDay.close, theDay.volume);
}

float MA(int day, int numDays, int stockNum){
	float total = 0.0f;
	int i;

	for(i = 0; i < numDays; i++){
		total += stockDays[stockNum][day-i].close;
	}

	return (total/numDays);
}

float EMA(int day, float percent, int stockNum, int whichEMA){
	//printf("Day:%i, Close:%f EMA:%f\n", day, stockDays[stockNum][day-1].close, (stockDays[stockNum][day-1].close * percent) + (EMA9[day-2] * (1 - percent)));

	return (stockDays[stockNum][day].close * percent) + (fIndVals[stockNum][whichEMA][day-1] * (1 - percent));
}



